import click
import sys
from api import archetypes
import constants
import os


@click.command()
@click.argument("NAME")
def docs(name):
    """Fetches the documentation url for the Marlin archetype or module NAME"""
    (archetype_details, error) = archetypes.get_archetype(archetype_name=name)
    if error is not None:
        click.echo(click.style("The requested entity does not exist.", fg="red"))
        sys.exit(1)

    click.echo("\n")
    click.echo(
        f"Documentation available at: {click.style(archetype_details['documentation_url'], fg='blue')}"
    )
    click.echo("\n")



import json

def json_merge(source, destination):
    """
    Add source json objects to destination json
    """
    with open(source, 'r') as s: 
        source_json = json.load(s)
    with open(destination, 'r') as d: 
        destination_json = json.load(d)
        
    def item_gen(source, dest):
      if isinstance(source, dict):
        for key, val in source.items():
          if key not in dest:
            dest[key] = val
          
          item_gen(val, dest[key])
      
    item_gen(source_json, destination_json)
    
    with open(destination, 'w') as d:
      json.dump(destination_json, d, indent=2)



from marlin_cli.commands.docs import docs
from marlin_cli.commands.init import init
from marlin_cli.commands.install import install
import click


@click.group()
@click.version_option("0.0.11")
def cli():
    pass


cli.add_command(init)
cli.add_command(docs)
cli.add_command(install)

if __name__ == "__main__":
    cli()



import click
import json
import sys
import os
import subprocess
import tarfile
import requests
import shutil
from api import modules, archetypes
from util import yaml_merge
from util import json_merge


def update_project(module_install_conf):
    click.echo("Updating project files")
    project_files = module_install_conf.get("project_updates")
    for source, destination in project_files.items():
        module_source = f"marlin-tmp/{source}"
        _, file_extension = os.path.splitext(module_source)
        match file_extension:
            case ".json":
                json_merge(module_source, destination)
            case ".yaml" | ".yml":
                yaml_merge(module_source, destination)
            case other:
                click.echo(
                    click.style(
                        f"Merge unsupported for file extension {other}", fg="red"
                    )
                )
                sys.exit(1)


def copy_source(module_install_conf):
    click.echo("Migrating module source code")
    source_targets = module_install_conf.get("raw_source_code")

    for source, destination in source_targets.items():
        shutil.copytree(f"marlin-tmp/{source}", destination, dirs_exist_ok=True)


def resolve_npm_deps(module_package_location):
    """
    Compare source and target package.json dependencies. Adds missing dependencies.
    """
    click.echo("Installing new npm dependencies")
    with open("package.json", "r") as f:
        package_json = json.load(f)
    with open(f"{module_package_location}/package.json", "r") as m:
        module_package_json = json.load(m)

    user_deps = package_json.get("dependencies")
    user_dev_deps = package_json.get("devDependencies")
    module_deps = module_package_json.get("dependencies")
    module_dev_deps = module_package_json.get("devDependencies")

    click.echo("Installing dependencies required by the module...")
    for dep, _ in module_dev_deps.items():
        if dep not in user_dev_deps:
            click.echo(
                click.style(f"Missing dev dependency {dep}. Installing...", fg="yellow")
            )
            subprocess.call(["npm", "install", dep, "--save-dev"])
    for dep, _ in module_deps.items():
        if dep not in user_deps:
            click.echo(
                click.style(f"Missing dependency {dep}. Installing...", fg="yellow")
            )
            subprocess.call(["npm", "install", dep])


def update_dependencies(archetype_name):
    (arch_conf, _) = archetypes.get_archetype(archetype_name)
    pm = arch_conf.get("package_manager")
    match pm:
        case "npm":
            if not os.path.exists("package.json"):
                click.echo(
                    click.style(
                        f"a package.json does not exist in this directory. Unable to install required dependencies",
                        fg="red",
                    )
                )
                sys.exit(1)
            resolve_npm_deps("marlin-tmp")
        case other:
            click.echo(
                click.style(
                    f"Unrecognized or Unsupported package manager {other}", fg="red"
                )
            )
            sys.exit(1)


@click.command()
@click.argument("module")
def install(module):
    """
    Installs the module MOUDLE in this project
    """
    # Require running from marlin root
    if not os.path.exists("marlinconf.json"):
        click.echo(
            click.style(f"marlinconf does not exist in this directory", fg="red")
        )
        sys.exit(1)

    (module_details, error) = modules.get_module(module_name=module)
    if error:
        click.echo(
            click.style(f"The requested module `{module}` does not exist.", fg="red")
        )
        sys.exit(1)
    click.echo(f"Found module: {module_details}")

    with open("marlinconf.json", "r") as f:
        marlinconf = json.load(f)

    project_archetype = marlinconf.get("archetype")
    click.echo(
        click.style(
            f"Installing {module} for archetype {project_archetype}", fg="green"
        )
    )

    # todo: fetch module and version
    repository = module_details.get("repository")
    url = f"https://api.github.com/repos/{repository.get('owner')}/{repository.get('repo_name')}/tarball/{repository.get('version')}"
    click.echo(f"Fetching module from {url}")
    response = requests.get(
        url=url,
        headers={
            "Accept": "application/vnd.github+json",
            "X-GitHub-Api-Version": "2022-11-28",
        },
    )
    click.echo(click.style(f"Received module source. Writing tarball", fg="green"))

    project_root = os.getcwd()
    # write temp dir
    if os.path.exists("marlin-tmp"):
        click.echo(
            click.style(
                f"A `marlin-tmp` directory already exists. Please remove it and try again.",
                fg="red",
            )
        )
        sys.exit(1)
    os.mkdir("marlin-tmp")

    # step into marlin-tmp for easy extract
    os.chdir("marlin-tmp")
    with open(f"{module}.tar", "wb") as fp:
        fp.write(response.content)

    archive = tarfile.open(f"{module}.tar")
    archive.extractall()
    os.remove(f"{module}.tar")
    # move contents to marlin-tmp
    extracted_dirname = os.listdir(os.getcwd())[0]
    shutil.copytree(
        os.path.join(os.getcwd(), extracted_dirname), os.getcwd(), dirs_exist_ok=True
    )
    shutil.rmtree(os.path.join(os.getcwd(), extracted_dirname))
    os.chdir(project_root)
    # step out of marlin-tmp

    with open("marlin-tmp/marlin-install.json", "r") as mi:
        module_install_conf = json.load(mi)

    click.echo(
        f"Installing with the marlin-install config\n\n{json.dumps(module_install_conf, indent=2)}\n"
    )

    # copy source files to their target dirs
    copy_source(module_install_conf)

    # update the project with other dependencies
    update_dependencies(project_archetype)
    update_project(module_install_conf)

    shutil.rmtree("marlin-tmp")

    click.echo(click.style(f"Successfully installed {module}", fg="green"))



import requests
import constants

def get_module(module_name):
    module = None
    error = None
    response = requests.get(url=f"{constants.API_URL}/modules/{module_name}")
    if response.ok:
        module = response.json()
    else:
        error = {"code": response.status_code, "message": response.content}
    return (module, error)

def list_modules():
    module_list = None
    error = None
    response = requests.get(url=f"{constants.API_URL}/modules")
    if response.ok:
        module_list = response.json()
    else:
        error = {"code": response.status_code, "message": response.content}
    return (module_list, error)



import click
import os
import sys
import json
import requests
import tarfile
import shutil
import subprocess
from git import Repo
from api import archetypes


@click.command()
@click.argument("archetype")
@click.argument("directory")
def init(archetype, directory):
    """
    Creates a new Marlin project based on ARCHETYPE at DIRECTORY
    """

    (archetype_details, error) = archetypes.get_archetype(archetype_name=archetype)
    if not (archetype_details):
        click.echo(click.style("The requested archetype does not exist.", fg="red"))
        sys.exit(1)
    click.echo(click.style(f"Found archetype: {archetype}", fg="green"))

    project_path = os.path.join(os.getcwd(), directory)
    if os.path.exists(project_path):
        click.echo(click.style(f"The directory {directory} already exists.", fg="red"))
        sys.exit(1)
    click.echo(click.style(f"Creating project at: {project_path}...", fg="green"))
    os.mkdir(project_path)
    os.chdir(project_path)

    repository = archetype_details.get("repository")
    url = f"https://api.github.com/repos/{repository.get('owner')}/{repository.get('repo_name')}/tarball/{repository.get('version')}"
    click.echo(click.style(f"Fetching archetype at {url}", fg="green"))
    response = requests.get(
        url=url,
        headers={
            "Accept": "application/vnd.github+json",
            "X-GitHub-Api-Version": "2022-11-28",
        },
    )
    click.echo(click.style(f"Writing archetype tarball", fg="green"))

    with open(f"{archetype}.tar", "wb") as fp:
        fp.write(response.content)

    archive = tarfile.open(f"{archetype}.tar")
    archive.extractall()
    os.remove(f"{archetype}.tar")
    extracted_dirname = os.listdir(project_path)[0]
    shutil.copytree(
        os.path.join(os.getcwd(), extracted_dirname), os.getcwd(), dirs_exist_ok=True
    )
    shutil.rmtree(os.path.join(os.getcwd(), extracted_dirname))

    click.echo(click.style(f"Creating marlinconf.json...", fg="green"))
    base_marlin_config = {"name": directory, "archetype": archetype}
    with open("marlinconf.json", "w") as fp:
        json.dump(base_marlin_config, fp, indent=2)

    click.echo(click.style(f"Initializing Git repository", fg="green"))
    repo = Repo.init(project_path)

    click.echo(click.style(f"Installing npm modules", fg="green"))
    subprocess.call(["npm", "install"])
    repo.git.add(all=True)
    repo.git.commit("-m", "Initial Commit")
    click.echo(
        click.style(
            f"Successfully initialized {archetype} at {project_path}", fg="green"
        )
    )



import requests
import constants


def get_archetype(archetype_name):
    archetype = None
    error = None
    response = requests.get(url=f"{constants.API_URL}/archetypes/{archetype_name}")
    if response.ok:
        archetype = response.json()
    else:
        error = {"code": response.status_code, "message": response.content}
    return (archetype, error)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['marlin_cli', 'marlin_cli.api', 'marlin_cli.commands', 'marlin_cli.util']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'gitpython>=3.1.30,<4.0.0',
 'pre-commit>=3.0.4,<4.0.0',
 'python-dotenv>=0.21.1,<0.22.0',
 'requests>=2.28.2,<3.0.0']

entry_points = \
{'console_scripts': ['marlin = marlin_cli.marlin:cli']}

setup_kwargs = {
    'name': 'marlin-cli',
    'version': '0.0.11',
    'description': '',
    'long_description': "# Marlin: The fastest path to modern web apps\n\n90% of software is the exact same; the final 10% is what differentiates a product. Marlin helps developers build the first 90% fast, so they can focus on what matters.\n\n## Installation\n\nMarlin can be built from source with the included build.sh script. Run the script and a single file executable will be written to `./dist/marlin`. Add this to your path or envoke it directly.\n\n## Documentation\n\nEventually we're going to put docs here\n\n## Contribute\n\nMarlin-cli uses pyenv for environment management and Poetry for dependcy management. These can be installed with the included `dev_setup.sh` script.\n\n\n## VSCode\n\nOpen VSCode and run the following commands in the terminal\n```\npoetry shell\npoetry env info --path | pbcopy\n```\n\nOpen the VSCode command palette with `cmd+shift+P` and search for `Python: select interpretor`. Open it, click `Enter interpreter path...` and paste the path copied from poetry above.",
    'author': 'George Cooper',
    'author_email': 'george@marlincode.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '==3.11.1',
}


setup(**setup_kwargs)
